#include <err.h>

#include "Halide.h"
#include "halide_image_io.h"

int
main(int argc, char *argv[])
{
	Halide::Var x("x"), y("y"), c("c");

	if (argc != 2)
		errx(1, "arguments");

	Halide::Image<u_int8_t> input = Halide::Tools::load_image(argv[1]);

	Halide::Expr clampx = Halide::clamp(x, 0, input.width() - 1);
	Halide::Expr clampy = Halide::clamp(y, 0, input.height() - 1);

	Halide::Func clamped("clamped");
	clamped(x, y, c) = input(clampx, clampy, c);

	Halide::Func u16in("u16in");
	u16in(x, y, c) = Halide::cast<u_int16_t>(clamped(x, y, c));

	Halide::Func blurx("blurx");
	blurx(x, y, c) =
	    (u16in(x - 1, y, c) + 2 * u16in(x, y, c) + u16in(x + 1, y, c)) / 4;

	Halide::Func blur("blur");
	blur(x, y, c) =
	    (blurx(x, y - 1, c) + 2 * blurx(x, y, c) + blurx(x, y + 1, c)) / 4;

	Halide::Func output("output");
	output(x, y, c) = Halide::cast<u_int8_t>(blur(x, y, c));

	Halide::Image<u_int8_t> result =
	    output.realize(input.width(), input.height(), 3);

	Halide::Tools::save_image(result, "output.png");
	return (0);
}
